import Hls from 'hls.js';
import { videoTypes, possibleQueryParameterExtensions } from './video-types';
export class VideoPlayer {
    constructor(mode, url, playerId, rate, exitOnEnd, loopOnEnd, container, zIndex, width, height) {
        this.pipMode = false;
        this._videoType = null;
        this._videoContainer = null;
        this._firstReadyToPlay = true;
        this._isEnded = false;
        this._videoRate = 1.0;
        this._videoExitOnEnd = true;
        this._videoLoopOnEnd = false;
        this._url = url;
        this._container = container;
        this._mode = mode;
        this._width = width ? width : 320;
        this._height = height ? height : 180;
        this._mode = mode;
        this._videoRate = rate;
        this._zIndex = zIndex ? zIndex : 1;
        this._playerId = playerId;
        this._videoExitOnEnd = exitOnEnd;
        this._videoLoopOnEnd = loopOnEnd;
    }
    async initialize() {
        // get the video type
        const urlVideoType = this._getVideoType();
        if (urlVideoType) {
            // style the container
            if (this._mode === 'fullscreen') {
                this._container.style.position = 'absolute';
                this._container.style.width = '100vw';
                this._container.style.height = '100vh';
            }
            if (this._mode === 'embedded') {
                this._container.style.position = 'relative';
                this._container.style.width = this._width.toString() + 'px';
                this._container.style.height = this._height.toString() + 'px';
            }
            this._container.style.left = '0';
            this._container.style.top = '0';
            this._container.style.display = 'flex';
            this._container.style.alignItems = 'center';
            this._container.style.justifyContent = 'center';
            this._container.style.backgroundColor = '#000000';
            this._container.style.zIndex = this._zIndex.toString();
            const width = this._mode === 'fullscreen'
                ? window.innerWidth /*this._container.offsetWidth*/
                : this._width;
            const height = this._mode === 'fullscreen'
                ? window.innerHeight /*this._container.offsetHeight*/
                : this._height;
            const xmlns = 'http://www.w3.org/2000/svg';
            const svg = document.createElementNS(xmlns, 'svg');
            svg.setAttributeNS(null, 'width', width.toString());
            svg.setAttributeNS(null, 'height', height.toString());
            const viewbox = '0 0 ' + width.toString() + ' ' + height.toString();
            svg.setAttributeNS(null, 'viewBox', viewbox);
            svg.style.zIndex = (this._zIndex + 1).toString();
            const rect = document.createElementNS(xmlns, 'rect');
            rect.setAttributeNS(null, 'x', '0');
            rect.setAttributeNS(null, 'y', '0');
            rect.setAttributeNS(null, 'width', width.toString());
            rect.setAttributeNS(null, 'height', height.toString());
            rect.setAttributeNS(null, 'fill', '#000000');
            svg.appendChild(rect);
            this._container.appendChild(svg);
            const heightVideo = (width * this._height) / this._width;
            this._videoContainer = document.createElement('div');
            this._videoContainer.style.position = 'absolute';
            this._videoContainer.style.left = '0';
            this._videoContainer.style.width = width.toString() + 'px';
            this._videoContainer.style.height = heightVideo.toString() + 'px';
            this._videoContainer.style.zIndex = (this._zIndex + 2).toString();
            this._container.appendChild(this._videoContainer);
            /*   Create Video Element */
            const isCreated = await this.createVideoElement(width, heightVideo);
            if (!isCreated) {
                this._createEvent('Exit', this._playerId, 'Video Error: failed to create the Video Element');
            }
        }
        else {
            this._createEvent('Exit', this._playerId, 'Url Error: type not supported');
        }
        return;
    }
    async createVideoElement(width, height) {
        this.videoEl = document.createElement('video');
        this.videoEl.controls = true;
        this.videoEl.style.zIndex = (this._zIndex + 3).toString();
        this.videoEl.style.width = `${width.toString()}px`;
        this.videoEl.style.height = `${height.toString()}px`;
        this.videoEl.playbackRate = this._videoRate;
        this._videoContainer.appendChild(this.videoEl);
        // set the player
        const isSet = await this._setPlayer();
        if (isSet) {
            this.videoEl.onended = async () => {
                this._isEnded = true;
                this.isPlaying = false;
                if (this.videoEl) {
                    this.videoEl.currentTime = 0;
                }
                if (this._videoExitOnEnd) {
                    if (this._mode === 'fullscreen') {
                        this._closeFullscreen();
                    }
                    this._createEvent('Ended', this._playerId);
                }
                else {
                    if (this._videoLoopOnEnd && this.videoEl != null) {
                        await this.videoEl.play();
                    }
                }
            };
            this.videoEl.oncanplay = async () => {
                if (this._firstReadyToPlay) {
                    this._createEvent('Ready', this._playerId);
                    if (this.videoEl != null) {
                        this.videoEl.muted = false;
                        if (this._mode === 'fullscreen')
                            await this.videoEl.play();
                        this._firstReadyToPlay = false;
                    }
                }
            };
            this.videoEl.onplay = () => {
                this.isPlaying = true;
                if (this._firstReadyToPlay)
                    this._firstReadyToPlay = false;
                this._createEvent('Play', this._playerId);
            };
            this.videoEl.onplaying = () => {
                this._createEvent('Playing', this._playerId);
            };
            this.videoEl.onpause = () => {
                this.isPlaying = false;
                this._createEvent('Pause', this._playerId);
            };
            if (this._mode === 'fullscreen') {
                // create the video player exit button
                const exitEl = document.createElement('button');
                exitEl.textContent = 'X';
                exitEl.style.position = 'absolute';
                exitEl.style.left = '1%';
                exitEl.style.top = '5%';
                exitEl.style.width = '5vmin';
                exitEl.style.padding = '0.5%';
                exitEl.style.fontSize = '1.2rem';
                exitEl.style.background = 'rgba(51,51,51,.4)';
                exitEl.style.color = '#fff';
                exitEl.style.visibility = 'hidden';
                exitEl.style.zIndex = (this._zIndex + 4).toString();
                exitEl.style.border = '1px solid rgba(51,51,51,.4)';
                exitEl.style.borderRadius = '20px';
                this._videoContainer.onclick = async () => {
                    this._initial = await this._doHide(exitEl, 3000);
                };
                this._videoContainer.ontouchstart = async () => {
                    this._initial = await this._doHide(exitEl, 3000);
                };
                this._videoContainer.onmousemove = async () => {
                    this._initial = await this._doHide(exitEl, 3000);
                };
                exitEl.onclick = () => {
                    this._createEvent('Exit', this._playerId);
                };
                exitEl.ontouchstart = () => {
                    this._createEvent('Exit', this._playerId);
                };
                this._videoContainer.appendChild(exitEl);
                this._initial = await this._doHide(exitEl, 3000);
                this._goFullscreen();
            }
        }
        return isSet;
    }
    async _goFullscreen() {
        if (this._container.mozRequestFullScreen) {
            /* Firefox */
            this._container.mozRequestFullScreen();
        }
        else if (this._container.webkitRequestFullscreen) {
            /* Chrome, Safari & Opera */
            this._container.webkitRequestFullscreen();
        }
        else if (this._container.msRequestFullscreen) {
            /* IE/Edge */
            this._container.msRequestFullscreen();
        }
        else if (this._container.requestFullscreen) {
            this._container.requestFullscreen();
        }
        return;
    }
    async _setPlayer() {
        return new Promise(resolve => {
            if (this.videoEl != null) {
                if (Hls.isSupported() && this._videoType === 'application/x-mpegURL') {
                    const hls = new Hls();
                    hls.loadSource(this._url);
                    hls.attachMedia(this.videoEl);
                    hls.once(Hls.Events.FRAG_PARSED, () => {
                        if (this.videoEl != null) {
                            this.videoEl.muted = true;
                            this.videoEl.crossOrigin = 'anonymous';
                            resolve(true);
                        }
                        else {
                            resolve(false);
                        }
                    });
                }
                else if (this._videoType === 'video/mp4') {
                    // CMAF (fMP4) && MP4
                    this.videoEl.src = this._url;
                    if (this._url.substring(0, 5) != 'https' &&
                        this._url.substring(0, 4) === 'http')
                        this.videoEl.crossOrigin = 'anonymous';
                    if (this._url.substring(0, 5) === 'https' ||
                        this._url.substring(0, 4) === 'http')
                        this.videoEl.muted = true;
                    resolve(true);
                }
                else {
                    // Not Supported
                    resolve(false);
                }
                this.videoEl.addEventListener('enterpictureinpicture', (event) => {
                    this.pipWindow = event.pictureInPictureWindow;
                    this.pipMode = true;
                    this._closeFullscreen();
                });
                this.videoEl.addEventListener('leavepictureinpicture', () => {
                    this.pipMode = false;
                    if (!this._isEnded) {
                        this._goFullscreen();
                        if (this.videoEl != null)
                            this.videoEl.play();
                    }
                });
            }
            else {
                resolve(false);
            }
        });
    }
    _getVideoType() {
        const sUrl = this._url ? this._url : '';
        if (sUrl != null && sUrl.length > 0) {
            Object.entries(videoTypes).forEach(([extension, mimeType]) => {
                // we search for dot + extension (e.g. `.mp4`) for URLs that have the extension in the filename
                // e.g. https://vimeo.com/?file=my-video.mp4
                const hasDotExtension = sUrl.match(new RegExp(`.(${extension})`, 'i'));
                if (hasDotExtension) {
                    return (this._videoType = mimeType);
                }
                // we search for the extension (e.g. `m3u8`) for URLs that might have the extension as a query parameter
                // e.g. https://youtube.com/?v=7894289374&type=m3u8
                const hasExtensionInUrl = sUrl.match(new RegExp(`(${extension})`, 'i'));
                if (hasExtensionInUrl) {
                    return (this._videoType = mimeType);
                }
            });
            // we check for not supported extensions for URLs that have the extension in the filename
            // e.g. https://vimeo.com/?file=not-supported-extension-video.mkv
            const hasNotSupportedDotExtension = sUrl.match(/\.(.*)/i);
            if (hasNotSupportedDotExtension) {
                return (this._videoType = null);
            }
            // we check for not supported extensions for URLs that might have the extension as a query parameter
            // e.g. https://youtube.com/?v=3982748927&filetype=mkv
            const hasNotSupportedExtensionInUrl = sUrl.match(new RegExp(`(${possibleQueryParameterExtensions.join('|')})\=+(.*)&?(?=&|$))`, 'i'));
            if (hasNotSupportedExtensionInUrl) {
                return (this._videoType = null);
            }
            // No extension found, then we assume it's 'mp4' (Match case for '')
            return 'video/mp4';
        }
        // URL was not defined, we return null
        return null;
    }
    async _doHide(exitEl, duration) {
        clearTimeout(this._initial);
        exitEl.style.visibility = 'visible';
        const initial = setTimeout(() => {
            exitEl.style.visibility = 'hidden';
        }, duration);
        return initial;
    }
    _createEvent(ev, playerId, msg) {
        const message = msg ? msg : null;
        let event;
        if (message != null) {
            event = new CustomEvent(`videoPlayer${ev}`, {
                detail: { fromPlayerId: playerId, message: message },
            });
        }
        else {
            const currentTime = this.videoEl ? this.videoEl.currentTime : 0;
            event = new CustomEvent(`videoPlayer${ev}`, {
                detail: { fromPlayerId: playerId, currentTime: currentTime },
            });
        }
        document.dispatchEvent(event);
    }
    _closeFullscreen() {
        const mydoc = document;
        const isInFullScreen = (mydoc.fullscreenElement && mydoc.fullscreenElement !== null) ||
            (mydoc.webkitFullscreenElement &&
                mydoc.webkitFullscreenElement !== null) ||
            (mydoc.mozFullScreenElement && mydoc.mozFullScreenElement !== null) ||
            (mydoc.msFullscreenElement && mydoc.msFullscreenElement !== null);
        if (isInFullScreen) {
            if (mydoc.mozCancelFullScreen) {
                mydoc.mozCancelFullScreen();
            }
            else if (mydoc.webkitExitFullscreen) {
                mydoc.webkitExitFullscreen();
            }
            else if (mydoc.msExitFullscreen) {
                mydoc.msExitFullscreen();
            }
            else if (mydoc.exitFullscreen) {
                mydoc.exitFullscreen();
            }
        }
    }
}
//# sourceMappingURL=videoplayer.js.map